/*
 * Copyright (C) 2005 Andre Noll <maan@systemlinux.org>
 *
 *     This program is free software; you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation; either version 2 of the License, or
 *     (at your option) any later version.
 *
 *     This program is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License
 *     along with this program; if not, write to the Free Software
 *     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
 */
#include "gcc-compat.h"
#include "para.h"

#include "recv.h"
#include "recv.cmdline.h"

extern int http_recv_init(struct receiver *r);
extern int ortp_recv_init(struct receiver *r);

struct gengetopt_args_info conf;

#if 0
static struct receiver receivers[] = {
	{
		.name = "http",
		.init = http_recv_init
	},
#ifdef HAVE_ORTP
	{
		.name = "ortp",
		.init = ortp_recv_init
	},
	{
		.name = NULL
	}
#endif
};
#endif

DEFINE_RECEIVER_ARRAY(receivers);

__printf(2,3) void para_log(int ll, char* fmt,...)
{
	va_list argp;

	/* ignore log message if loglevel is not high enough */
	if (ll < conf.loglevel_arg)
		return;
	va_start(argp, fmt);
	vfprintf(stderr, fmt, argp);
	va_end(argp);
}

int main(int argc, char *argv[])
{
	int i, ret, eof = 0, max, r_opened = 0;
	struct timeval timeout;
	struct	receiver *r = NULL;
	fd_set rfds, wfds;
	struct receiver_node *rn = para_calloc(sizeof(struct receiver_node));

	cmdline_parser(argc, argv, &conf);

	for (i = 0; receivers[i].name; i++) {
		if (strcmp(receivers[i].name, conf.receiver_arg))
			continue;
		r = &receivers[i];
		PARA_INFO_LOG("using the %s receiver\n", r->name);
		break;
	}
	ret = -E_SYNTAX;
	if (!r)
		goto out;
	ret = r->init(r);
	if (ret < 0)
		goto out;
	rn->receiver = r;
	rn->host = conf.host_arg;
	rn->port = conf.port_arg;
	if (conf.header_given)
		rn->flags |= HEADER_WAIT;
	rn->chunk_tv.tv_sec = 0;
	rn->chunk_tv.tv_usec = conf.chunk_time_arg;
	ret = r->open(rn);
	if (ret < 0)
		goto out;
	r_opened = 1;
recv:
	FD_ZERO(&rfds);
	FD_ZERO(&wfds);
	timeout.tv_sec = 0;
	timeout.tv_usec = 1000 * 1000;
	max = -1;
	ret = r->pre_select(rn, &rfds, &wfds, &timeout);
	max = MAX(max, ret);

	PARA_DEBUG_LOG("timeout: %lums\n", tv2ms(&timeout));
	ret = select(max + 1, &rfds, &wfds, NULL, &timeout);
	if (ret < 0) {
		if (errno == EINTR || errno == EAGAIN)
			goto recv;
		ret = -E_SELECT;
		goto out;
	}
	ret = r->post_select(rn, ret, &rfds, &wfds);
	if (ret < 0)
		goto out;
	if (!ret)
		eof = 1;
	if (!rn->loaded) {
		if (eof)
			goto out;
		goto recv;
	}
	ret = write(STDOUT_FILENO, rn->buf, rn->loaded);
	PARA_DEBUG_LOG("wrote %d/%d\n", ret, rn->loaded);
	if (ret < 0) {
		ret = -E_WRITE;
		goto out;
	}
	if (ret != rn->loaded) {
		PARA_INFO_LOG("short write %d/%d\n", ret, rn->loaded);
		memmove(rn->buf, rn->buf + ret, rn->loaded - ret);
	}
	rn->loaded -= ret;
	if (rn->loaded || !eof)
		goto recv;
out:
	PARA_NOTICE_LOG("exit code: %d\n", ret);
	if (r_opened)
		r->close(rn);
	if (r)
		r->shutdown();
	return ret;
}
